/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AlertInterface, AlertLevels, TestableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import { EncodeDecodeUtils } from "@wso2is/core/utils";
import { CodeEditor, GenericIcon, Heading, LinkButton } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { Button, Card, Divider, Icon, Message } from "semantic-ui-react";
import { ReactComponent as AngularLogo } from "./assets/angular-logo.svg";
import DownloadIllustration from "./assets/config-copying-illustration.svg";
import { ReactComponent as JavaScriptLogo } from "./assets/javascript-logo.svg";
import { SDKMeta } from "./meta";
import {
    ApplicationInterface,
    ApplicationManagementUtils,
    ApplicationTemplateInterface,
    OIDCDataInterface,
    SupportedAuthProtocolTypes,
    updateAuthProtocolConfig
} from "../../../../features/applications";
import { AppState } from "../../../../features/core";
import { Config } from "../../../../features/core/configs";
import { ReactComponent as DownloadArrow } from "../../../assets/images/icons/download-arrow.svg";
import ReactLogoDataURL, { ReactComponent as ReactLogo } from "../../../assets/images/icons/react-icon.svg";
import { VerticalStepper } from "../../../components/component-extensions/application/vertical-stepper";

interface TryoutSamplesPropsInterface extends TestableComponentInterface {
    application: ApplicationInterface;
    template: ApplicationTemplateInterface;
    inboundProtocolConfig: any;
}

enum SampleType {
    ANGULAR = "Angular",
    REACT = "React",
    JAVASCRIPT = "JavaScript"
}

/**
 * Tryout Samples of Single Page Applications.
 *
 * @param {TryoutSamplesPropsInterface} props - Props injected into the component.
 * @return {React.ReactElement}
 */
export const TryoutSamples: FunctionComponent<TryoutSamplesPropsInterface> = (
    props: TryoutSamplesPropsInterface
): ReactElement => {

    const {
        application,
        inboundProtocolConfig,
        [ "data-testid" ]: testId
    } = props;

    const dispatch = useDispatch();

    const helpPanelVisibilityGlobalState: boolean = useSelector((state: AppState) => state.helpPanel.visibility);

    const [ authConfig, setAuthConfig ] = useState(undefined);

    const [ selectedSample, setSelectedSample ] = useState(undefined);
    const [ callbacksUpdated, setCallbacksUpdated ] = useState<boolean>(false);

    useEffect(() => {
        if (inboundProtocolConfig == undefined) {
            return;
        }

        const configs = {
            clientID: inboundProtocolConfig.oidc.clientId,
            serverOrigin: Config.getDeploymentConfig().serverHost,
            signInRedirectURL: "http://localhost:3000",
            signOutRedirectURL: "http://localhost:3000"
        };

        setAuthConfig(configs);
    }, [ inboundProtocolConfig ]);

    const checkCallbacks = (urlToCheck: string) => {
        const urlArray = inboundProtocolConfig.oidc
            ? EncodeDecodeUtils.decodeURLRegex(inboundProtocolConfig.oidc.callbackURLs[ 0 ])
            : [];

        if (!urlArray || !Array.isArray(urlArray) || urlArray.length < 1) {

            return false;
        }

        return urlArray.includes(urlToCheck);
    };

    const generateIntegrateCode = (language: SampleType) => {

        if (!authConfig) {
            return null;
        }

        if (language === SampleType.REACT) {
            return `{
    "clientID": "${ authConfig.clientID }",
    "serverOrigin": "${ authConfig.serverOrigin }",
    "signInRedirectURL": "${ authConfig.signInRedirectURL }",
    "signOutRedirectURL": "${ authConfig.signOutRedirectURL }"
}`;
        }

        return `const authConfig = {
    clientID: "${ authConfig.clientID }",
    signInRedirectURL: "${ authConfig.signInRedirectURL }",
    signOutRedirectURL: "${ authConfig.signOutRedirectURL }",
    serverOrigin: "${ authConfig.serverOrigin }",
    enablePKCE: true
};`;

    };

    const downloadConfigFile = (): void => {
        const blob = new Blob(
            [ JSON.stringify(authConfig, null, 2) ],
            { type: "application/json" }
        );
        const url = window.URL.createObjectURL(blob);
        const a = document.createElement("a");
        a.style.display = "none";
        a.href = url;
        a.download = "config.json";
        document.body.appendChild(a);
        a.click();
        window.URL.revokeObjectURL(url);
    };

    const handleAddCallback = (url: string) => {

        const configuredCallbacks: any = ApplicationManagementUtils.buildCallBackURLWithSeparator(
            inboundProtocolConfig.oidc.callbackURLs[ 0 ]).split(",");

        const shouldUpdateCallbacks: boolean = !configuredCallbacks.includes(url);
        const shouldUpdateAllowedOrigins: boolean = !inboundProtocolConfig.oidc.allowedOrigins.includes(url);

        const body: OIDCDataInterface = {
            ...inboundProtocolConfig.oidc,
            allowedOrigins: shouldUpdateAllowedOrigins
                ? [ ...inboundProtocolConfig.oidc.allowedOrigins, url ]
                : inboundProtocolConfig.oidc.allowedOrigins,
            callbackURLs: shouldUpdateCallbacks
                ? [ ApplicationManagementUtils.buildCallBackUrlWithRegExp([ ...configuredCallbacks, url ].join(",")) ]
                : inboundProtocolConfig.oidc.callbackURLs
        };

        const resolveAlertContent = () => {

            if (shouldUpdateAllowedOrigins && shouldUpdateCallbacks) {
                return {
                    description: "Successfully updated the URLs in the application.",
                    level: AlertLevels.SUCCESS,
                    message: "Updated the URLs"
                };
            } else if (shouldUpdateCallbacks) {
                return {
                    description: "Successfully updated the Allowed Origins.",
                    level: AlertLevels.SUCCESS,
                    message: "Updated the URLs"
                };
            }

            return {
                description: "Successfully updated the Callback URLs in the application.",
                level: AlertLevels.SUCCESS,
                message: "Updated the URLs"
            };
        };

        updateAuthProtocolConfig(application.id, body, SupportedAuthProtocolTypes.OIDC)
            .then(() => {
                setCallbacksUpdated(true);

                dispatch(addAlert<AlertInterface>(resolveAlertContent()));
            })
            .catch(() => {
                dispatch(addAlert<AlertInterface>({
                    description: "An error occurred while updating the application.",
                    level: AlertLevels.ERROR,
                    message: "Error occurred"
                }));
            });
    };

    const renderSampleDownloadButton = (sample: SampleType) => {

        if (sample === SampleType.REACT) {
            return (
                <>
                    <p>Click on the button bellow to download the sample. Click <a
                        href={ SDKMeta.javascript.samples.react.repository }
                        target="_black">here</a> to check the source on GitHub.</p>
                    <Button
                        data-testid={ `${ testId }-download-react-sample` }
                        className="sample-download mr-3"
                        basic
                        onClick={ () => window.open(SDKMeta.javascript.samples.react.artifact, "") }
                    >
                        <GenericIcon
                            transparent
                            icon={ ReactLogoDataURL }
                            size="mini"
                            spaced="right"
                            floated="left"
                        />
                        Download { sample } Sample
                        <GenericIcon
                            transparent
                            icon={ DownloadArrow }
                            size="mini"
                            fill="default"
                            spaced="left"
                            floated="right"
                        />
                    </Button>
                    <LinkButton
                        compact
                        onClick={ () => window.open(SDKMeta.javascript.samples.react.repository, "") }
                    >
                        Explore more samples
                        <Icon className="ml-1" name="external"/>
                    </LinkButton>
                </>
            );
        } else if (sample === SampleType.JAVASCRIPT) {
            return (
                <>
                    <p>Click on the button bellow to download the sample. Click <a
                        href={ SDKMeta.javascript.samples.javascript.repository }
                        target="_black">here</a> to check the source on GitHub.</p>
                    <Button
                        data-testid={ `${ testId }-download-js-sample` }
                        className="sample-download"
                        basic
                        onClick={ () => window.open(SDKMeta.javascript.samples.javascript.artifact, "") }
                    >
                        <GenericIcon
                            transparent
                            icon={ JavaScriptLogo }
                            size="mini"
                            spaced="right"
                            floated="left"
                        />
                        Download { selectedSample } Sample
                        <GenericIcon
                            transparent
                            icon={ DownloadArrow }
                            size="mini"
                            fill="default"
                            spaced="left"
                            floated="right"
                        />
                    </Button>
                    <LinkButton
                        compact
                        onClick={ () => window.open(SDKMeta.javascript.samples.javascript.repository, "") }
                    >
                        Explore more samples
                        <Icon className="ml-1" name="external"/>
                    </LinkButton>
                </>
            );
        } else if (sample === SampleType.ANGULAR) {
            return (
                <>
                    <p>Click on the button bellow to download the sample. Click <a href="" target="_black">here</a>
                        to check the source on GitHub.</p>
                    <Button
                        data-testid={ `${ testId }-download-angular-sample` }
                        className="sample-download mr-3"
                        basic
                        onClick={ () => window.open("", "") }
                    >
                        <GenericIcon
                            transparent
                            icon={ AngularLogo }
                            size="mini"
                            spaced="right"
                            floated="left"
                        />
                        Download { sample } Sample
                        <GenericIcon
                            transparent
                            icon={ DownloadArrow }
                            size="mini"
                            fill="default"
                            spaced="left"
                            floated="right"
                        />
                    </Button>
                    <LinkButton
                        compact
                        onClick={ () => window.open(SDKMeta.javascript.samples.react.repository, "") }
                    >
                        Explore more samples
                        <Icon className="ml-1" name="external"/>
                    </LinkButton>
                </>
            );
        }
    };

    const generateConfigureStep = (sample: SampleType) => {

        if (sample === SampleType.REACT) {
            return (
                <>
                    <Divider hidden className="x1"/>
                    <Heading as="h4">
                        Download the Configuration
                    </Heading>
                    <p>Click on the following button to downlaod the configuration file.</p>
                    <Button
                        onClick={ downloadConfigFile }
                        className="download-config"
                        size='small'
                        primary
                        data-testid={ `${ testId }-download-config` }
                    >
                        Download config.json
                    </Button>

                    <Divider hidden/>
                    <Heading as="h4">
                        Extract the Configuration File
                    </Heading>
                    <p>Extract downloaded sample and replace the <strong>src/config.json</strong> file with the
                        below configuration file</p>
                    <GenericIcon
                        icon={ DownloadIllustration }
                        size="large"
                        transparent
                        className="download-illustration"
                    />
                    <Divider hidden/>
                    <p>or copy and paste below configuration in <strong>src/config.json</strong>.</p>
                    <div className="code-segment">
                        <div className="action-buttons">
                            <Button basic size='tiny' className="copy-config" icon>
                                <Icon name='copy'/>
                            </Button>
                        </div>
                        <CodeEditor
                            showLineNumbers={ false }
                            language="json"
                            sourceCode={ generateIntegrateCode(SampleType.REACT) }
                            beautify={ true }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="light"
                            readOnly={ true }
                        />
                    </div>
                </>
            );
        } else if (sample === SampleType.JAVASCRIPT) {
            return (
                <>
                    <p>Open the <strong>index.html</strong> file located at the root of
                        the project & scroll down to the <strong>{ "<script>" }</strong> tag at the end of the
                        <strong>body</strong> tag and find `authConfig` object.</p>
                    {
                        !callbacksUpdated && !checkCallbacks("http://localhost:3000")
                            ? (
                                <Message warning>
                                    <p>
                                        In order to try out the sample, you need to
                                        add <strong>http://localhost:3000</strong> to <strong>{ " " }
                                        allowed Redirect URLs</strong>
                                        <Button color="orange" floated="right"
                                                onClick={ () => handleAddCallback("http://localhost:3000") }>Add
                                            Now</Button>
                                    </p>
                                </Message>
                            )
                            : null
                    }
                    <Divider hidden/>
                    <p>Change the configuration object as follows.</p>
                    <div className="code-segment">
                        <div className="action-buttons">
                            <Button basic size='tiny' className="copy-config" icon>
                                <Icon name='copy'/>
                            </Button>
                        </div>
                        <CodeEditor
                            showLineNumbers={ false }
                            language="javascript"
                            sourceCode={ generateIntegrateCode(SampleType.JAVASCRIPT) }
                            beautify={ true }
                            theme={ "light" }
                            options={ {
                                lineWrapping: true
                            } }
                            readOnly={ true }
                        />
                    </div>
                </>
            );
        } else if (sample === SampleType.ANGULAR) {
            return (
                <>
                    <Divider hidden className="x1"/>
                    <Heading as="h4">
                        Download the Configuration
                    </Heading>
                    <p>Click on the following button to downlaod the configuration file.</p>
                    <Button
                        onClick={ downloadConfigFile }
                        className="download-config"
                        size='small'
                        primary
                        data-testid={ `${ testId }-download-config` }
                    >
                        Download config.json
                    </Button>

                    <Divider hidden/>
                    <Heading as="h4">
                        Extract the Configuration File
                    </Heading>
                    <p>Extract downloaded sample and replace the <strong>src/config.json</strong> file with the
                        below configuration file</p>
                    <GenericIcon
                        icon={ DownloadIllustration }
                        size="large"
                        transparent
                        className="download-illustration"
                    />
                    <Divider hidden/>
                    <p>or copy and paste below configuration in <strong>src/config.json</strong>.</p>
                    <div className="code-segment">
                        <div className="action-buttons">
                            <Button basic size='tiny' className="copy-config" icon>
                                <Icon name='copy'/>
                            </Button>
                        </div>
                        <CodeEditor
                            showLineNumbers={ false }
                            language="json"
                            sourceCode={ generateIntegrateCode(SampleType.ANGULAR) }
                            beautify={ true }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="light"
                            readOnly={ true }
                        />
                    </div>
                </>
            );
        }

        return null;
    };

    const webAppContent = [
        {
            stepContent: (
                <>
                    <p>Choose the technology that your application is built on.</p>
                    <Card.Group className="mt-3" itemsPerRow={ helpPanelVisibilityGlobalState ? 5 : 9 }>
                        <Card
                            raised={ false }
                            data-testid={ `${ testId }-technology-react` }
                            className={
                                `basic-card tech-selection ${
                                    (selectedSample === SampleType.REACT)
                                        ? "selected"
                                        : ""
                                    } `
                            }
                            onClick={ () => {
                                setSelectedSample(SampleType.REACT);
                            } }
                        >
                            <Card.Content textAlign="center">
                                <ReactLogo />
                                <Card.Description>
                                    React
                                </Card.Description>
                            </Card.Content>
                        </Card>
                        <Card
                            raised={ false }
                            data-testid={ `${ testId }-technology-angular` }
                            className={
                                `basic-card tech-selection ${
                                    (selectedSample === SampleType.ANGULAR)
                                        ? "selected"
                                        : ""
                                    } `
                            }
                            onClick={ () => {
                                setSelectedSample(SampleType.ANGULAR);
                            } }
                        >
                            <Card.Content textAlign="center">
                                <AngularLogo />
                                <Card.Description>
                                    Angular
                                </Card.Description>
                            </Card.Content>
                        </Card>
                        <Card
                            raised={ false }
                            data-testid={ `${ testId }-technology-javascript` }
                            className={
                                `basic-card tech-selection ${
                                    (selectedSample === SampleType.JAVASCRIPT)
                                        ? "selected"
                                        : ""
                                    } `
                            }
                            onClick={ () => {
                                setSelectedSample(SampleType.JAVASCRIPT);
                            } }
                        >
                            <Card.Content textAlign="center">
                                <JavaScriptLogo />
                                <Card.Description>
                                    JavaScript
                                </Card.Description>
                            </Card.Content>
                        </Card>
                    </Card.Group>
                </>
            ),
            stepTitle: `Select technology ${ selectedSample ? "(" + selectedSample + ")" : "" }`
        },
        {
            stepContent: renderSampleDownloadButton(selectedSample),
            stepTitle: "Download sample"
        },
        {
            stepContent: (
                <>
                    {
                        !callbacksUpdated
                        && !(checkCallbacks("http://localhost:3000")
                            && checkCallbacks("http://localhost:3000"))
                            ? (
                                <Message warning>
                                    <p>
                                        In order to try out the sample, you need to
                                        add <strong>http://localhost:3000</strong> to <strong>Redirect
                                        URLs</strong>
                                        <Button color="orange" floated="right"
                                                onClick={ () => handleAddCallback("http://localhost:3000") }>Add
                                            Now</Button>
                                    </p>
                                </Message>
                            )
                            : null
                    }
                    { generateConfigureStep(selectedSample) }
                </>
            ),
            stepTitle: "Configure the sample"
        },
        {
            stepContent: (
                <>
                    <p>NOTE: Make sure you have <strong>Node.js</strong> and <strong>npm</strong> installed on your
                        environment.</p>
                    <p className="mt-2">
                        Run the below command at the root of the project to fire up the development server. The app
                        should open on <a>http://localhost:3000</a>.
                    </p>
                    <code>
                        npm install && npm start
                    </code>
                </>
            ),
            stepTitle: "Run the sample"
        }
    ];

    return (
        <VerticalStepper
            stepContent={ webAppContent }
            isNextEnabled={ selectedSample }
            isSidePanelOpen
        />
    );
};
