/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { Heading } from "@wso2is/react-components";
import isEmpty from "lodash/isEmpty";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { Card, Grid, Header, Radio } from "semantic-ui-react";
import {
    ApplicationInterface,
    ApplicationManagementConstants,
    ApplicationManagementUtils,
    InboundProtocolListItemInterface,
    OIDCApplicationConfigurationInterface,
    SAMLApplicationConfigurationInterface
} from "../../../../features/applications";
import { AppState } from "../../../../features/core";
import { QuickStartModes } from "../../../application-templates/shared";

/**
 * Proptypes for the applications help panel overview component.
 */
interface HelpPanelOverviewPropsInterface extends TestableComponentInterface {
    inboundProtocols?: InboundProtocolListItemInterface[];
    application?: ApplicationInterface;
    applicationType: string;
    handleIntegrateSelection?: (selection: string) => void;
}

/**
 * Applications Help Panel Overview Component.
 * TODO: Add localization support. (https://github.com/wso2-enterprise/asgardeo-product/issues/209)
 *
 * @param {HelpPanelOverviewPropsInterface} props - Props injected to the component.
 * @return {React.ReactElement}
 */
export const HelpPanelOverview: FunctionComponent<HelpPanelOverviewPropsInterface> = (
    props: HelpPanelOverviewPropsInterface
): ReactElement => {

    const urlSearchParams: URLSearchParams = new URLSearchParams(location.search);
    const isNewApp: boolean = urlSearchParams.get(
        ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_KEY
    ) === ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_VALUE;

    const oidcConfigurations: OIDCApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.oidcConfigurations);
    const samlConfigurations: SAMLApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.samlConfigurations);
    const { t } = useTranslation();

    const {
        application,
        applicationType,
        inboundProtocols,
        handleIntegrateSelection
    } = props;

    const [ isOIDC, setIsOIDC ] = useState<boolean>(false);
    const [ isSAML, setIsSAML ] = useState<boolean>(false);
    const [ isOIDCConfigsLoading, setOIDCConfigsLoading ] = useState<boolean>(false);
    const [ isSAMLConfigsLoading, setSAMLConfigsLoading ] = useState<boolean>(false);
    const [ selectedIntegration, setSelectedIntegration ] = useState<QuickStartModes>(QuickStartModes.INTEGRATE);

    useEffect(() => {
        if (inboundProtocols == undefined) {
            return;
        }

        if (inboundProtocols.length > 0) {
            inboundProtocols.map((protocol) => {
                if (protocol.type == "oauth2") {
                    setIsOIDC(true);
                } else if (protocol.type == "samlsso") {
                    setIsSAML(true);
                }
            });
        }

    }, [ inboundProtocols ]);

    useEffect(() => {
        if (oidcConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getOIDCApplicationMeta()
            .finally(() => {
                setOIDCConfigsLoading(false);
            });
    }, [ oidcConfigurations ]);

    useEffect(() => {
        if (samlConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getSAMLApplicationMeta()
            .finally(() => {
                setSAMLConfigsLoading(false);
            });
    }, [ samlConfigurations ]);

    /**
     * Resolves application name to be shown at initial creation.
     * Suffixes the `application` part if required.
     *
     * @param {string} name - App name.
     * @return {React.ReactElement}
     */
    const resolveApplicationName = (name: string): ReactElement => {

        const nameTokens: string[] = name.split(" ");
        const matchers: string[] = [ "app", "application" ];
        let requireNameSuffix: boolean = true;

        if (Array.isArray(nameTokens) && !isEmpty(nameTokens)) {
            const ending: string = nameTokens[ nameTokens.length - 1 ];

            for (const matcher of matchers) {
                if (ending.toLocaleLowerCase() === matcher) {
                    requireNameSuffix = false;
                    break;
                }
            }
        }

        return (
            <>
                <strong>{ application.name }</strong> { requireNameSuffix && <> application</> } has been registered!
            </>
        );
    };

    return (
        <>
            <Grid>
                {
                    isNewApp
                        ? (
                            <Grid.Row>
                                <Grid.Column width={ 12 }>
                                    <Heading as="h3" className="mt-3 mb-0">
                                        { resolveApplicationName(application.name) }
                                    </Heading>
                                    <Header.Subheader className="font-weight-lighter">
                                        Please select one of the following methods to get started.
                                    </Header.Subheader>
                                </Grid.Column>
                            </Grid.Row>
                        )
                        : (
                            <Grid.Row>
                                <Grid.Column width={ 12 }>
                                    <Header.Subheader className="font-weight-lighter">
                                        Please select one of the following methods to get started.
                                    </Header.Subheader>
                                </Grid.Column>
                            </Grid.Row>
                        )
                }
                {
                    applicationType
                        ? (
                            <>
                                <Grid.Row>
                                    <Grid.Column width={ 8 }>
                                        <Card
                                            className={
                                                `selection-card ${ selectedIntegration === QuickStartModes.INTEGRATE
                                                    ? "card-selected"
                                                    : ""
                                                    }`
                                            }
                                            fluid
                                        >
                                            <Card.Content
                                                onClick={ () => {
                                                    setSelectedIntegration(QuickStartModes.INTEGRATE);
                                                    handleIntegrateSelection(QuickStartModes.INTEGRATE);
                                                } }
                                                className="selection-card-content"
                                            >
                                                <div className="integrate-radio">
                                                    <Radio
                                                        checked={ selectedIntegration === QuickStartModes.INTEGRATE }
                                                    />
                                                </div>
                                                <div className="card-content">
                                                    <Card.Header>
                                                        <span>Integrate</span> with your application
                                                    </Card.Header>
                                                    <Card.Meta>
                                                        <span>
                                                            Follow the below instruction to integrate your own
                                                            application
                                                        </span>
                                                    </Card.Meta>
                                                </div>
                                            </Card.Content>
                                        </Card>
                                    </Grid.Column>
                                    <Grid.Column width={ 8 }>
                                        <Card
                                            fluid
                                            className={
                                                `selection-card ${ selectedIntegration === QuickStartModes.SAMPLES
                                                    ? "card-selected"
                                                    : "" }`
                                            }
                                        >
                                            <Card.Content
                                                onClick={ () => {
                                                    setSelectedIntegration(QuickStartModes.SAMPLES);
                                                    handleIntegrateSelection(QuickStartModes.SAMPLES);
                                                } }
                                                className="selection-card-content"
                                            >
                                                <div className="integrate-radio">
                                                    <Radio
                                                        checked={ selectedIntegration === QuickStartModes.SAMPLES }
                                                    />
                                                </div>
                                                <div className="card-content">
                                                    <Card.Header><span>Try out </span>with a sample</Card.Header>
                                                    <Card.Meta>
                                                        <span>
                                                            Use our pre configured samples and try out the
                                                            authentication.
                                                        </span>
                                                    </Card.Meta>
                                                </div>
                                            </Card.Content>
                                        </Card>
                                    </Grid.Column>
                                </Grid.Row>
                            </>
                        )
                        : null
                }
            </Grid>
        </>
    );
};
